import sys
import xml.etree.ElementTree as ET
from internal.ErrorHandling import *
from scipy import interpolate
from scipy import stats
import numpy as np
import math

class ComputeLikelihood:
    """Compute the likelihood from experimental mu and user mu."""

    def __init__(self):
        """Compute the likelihood from experimental mu and user mu."""
        self.likelihoods = []

    def compute(self, exp_mu, user_mu):
        """compute -2log(L)"""
        l = 0.
        ndf=0

        for mu in exp_mu:
            # ===============
            # compute user mu value scaled to efficiencies
            # ===============
            user_mu_effscaled = {}
        
            user_mu_effscaled["x"] = 0.
            
            for (prod,decay), eff_prod in mu["eff"]["x"].items():
                user_mu_effscaled["x"] += eff_prod*user_mu[prod,decay]

            if mu["dim"] == 2:
                user_mu_effscaled["y"] = 0.

                for (prod,decay), eff_prod in mu["eff"]["y"].items():
                    user_mu_effscaled["y"] += eff_prod*user_mu[prod, decay]
    
            # ===============
            # likelihood computation in case of a type="normal" (Gaussian)
            # ===============
            if mu["type"] == "n":
                if mu["dim"] == 1:
                    if user_mu_effscaled["x"] < mu["bestfit"]["x"]:
                        unc = mu["param"]["uncertainty"]["left"]
                    else:
                        unc = mu["param"]["uncertainty"]["right"]
                    cur_l = (mu["bestfit"]["x"] - user_mu_effscaled["x"])**2 / unc**2
                    ndf += 1
                    
                elif mu["dim"] == 2:
                    a = mu["param"]["a"]
                    b = mu["param"]["b"]
                    c = mu["param"]["c"]
    
                    cur_l = a*(mu["bestfit"]["x"] - user_mu_effscaled["x"])**2
                    cur_l += c*(mu["bestfit"]["y"] - user_mu_effscaled["y"])**2
                    cur_l += 2*b*(mu["bestfit"]["x"] - user_mu_effscaled["x"])*(mu["bestfit"]["y"] - user_mu_effscaled["y"])
                    ndf += 2
            
            if mu["type"] == "f":
                if mu["dim"] == 1:
                    cur_l = mu["Lxy"](user_mu_effscaled["x"]) - mu["LChi2min"]
                    ndf += 1
                elif mu["dim"] == 2:
                    cur_l = mu["Lxy"](user_mu_effscaled["x"], user_mu_effscaled["y"])[0][0] - mu["LChi2min"]
                    ndf += 2

            l += cur_l
            self.likelihoods.append(
            {"experiment": mu["experiment"], "source": mu["source"],
            "sqrts": mu["sqrts"], "dim": mu["dim"],
            "type": mu["type"], "eff": mu["eff"], "l": cur_l})

        pvalue = 1 - stats.chi2.cdf(l, ndf)
        
        return l, ndf, pvalue
