import sys
import math
#from BR_SM import *

class ComputeMuFromReducedCouplings:
    
    def __init__(self, redC, extraBR, mh, precision, BR):

        self.redC = redC
        self.extraBR = extraBR
        self.mh = mh
        self.precision = precision
        self.BR = BR
        self.mu = self.GetMu()
    
    def GetMu(self):
        
        if self.precision == "LO":
            for key, val in self.redC.items():
                if key == "tt":
                    self.redC[key] = math.sqrt(val.real**2 + 1*val.imag**2)
                else:
                    self.redC[key] = abs(val)

        if self.precision == "BEST-QCD":
            for key, val in self.redC.items():
                self.redC[key] = val.real
                
        BR_SM_bb = self.BR["BR_bb"]
        BR_SM_cc = self.BR["BR_cc"]
        BR_SM_tautau = self.BR["BR_tautau"]
        BR_SM_mumu = self.BR["BR_mumu"]
        BR_SM_ss = self.BR["BR_ss"]
        BR_SM_WW = self.BR["BR_WW"]
        BR_SM_ZZ = self.BR["BR_ZZ"]
        BR_SM_GaGa = self.BR["BR_GaGa"]
        BR_SM_gg = self.BR["BR_gg"]
        BR_SM_ZGa = self.BR["BR_ZGa"]
        
        width_SM = BR_SM_bb + BR_SM_cc + BR_SM_tautau + 0*BR_SM_mumu + 0*BR_SM_ss + BR_SM_WW + BR_SM_ZZ + BR_SM_GaGa + BR_SM_gg + BR_SM_ZGa



        width_bb = BR_SM_bb * self.redC["bb"]**2
        width_cc = BR_SM_cc * self.redC["cc"]**2
        width_tautau = BR_SM_tautau * self.redC["tautau"]**2
        width_WW = BR_SM_WW * self.redC["WW"]**2
        width_ZZ = BR_SM_ZZ * self.redC["ZZ"]**2
        width_GaGa = BR_SM_GaGa * self.redC["gammagamma"]**2
        width_gg = BR_SM_gg * self.redC["gg_decay"]**2
        width_ZGa = BR_SM_ZGa * self.redC["Zgamma"]**2
        width_mumu = BR_SM_mumu * 1**2
        width_ss = BR_SM_ss * 1**2
        
        
        
        width = width_bb + width_cc + width_tautau + 0*width_mumu + 0*width_ss + width_WW + width_ZZ + width_GaGa + width_gg + width_ZGa
        width = width/width_SM
        
        
        inv_und = (1.-self.extraBR["invisible"])*(1.-self.extraBR["undetected"])

        redBR_bb = inv_und * width_bb/width/BR_SM_bb
        redBR_cc = inv_und * width_cc/width/BR_SM_cc
        redBR_tautau = inv_und * width_tautau/width/BR_SM_tautau
        redBR_WW = inv_und * width_WW/width/BR_SM_WW
        redBR_ZZ = inv_und * width_ZZ/width/BR_SM_ZZ
        redBR_GaGa = inv_und * width_GaGa/width/BR_SM_GaGa
        redBR_gg = inv_und * width_gg/width/BR_SM_gg
        redBR_ZGa = inv_und * width_ZGa/width/BR_SM_ZGa


        if self.precision == "LO":
            # decay=production @ LO
            ggH_bb = self.redC["gg_decay"]**2 * redBR_bb
            ggH_GaGa = self.redC["gg_decay"]**2 * redBR_GaGa
            ggH_WW = self.redC["gg_decay"]**2 * redBR_WW
            ggH_ZZ = self.redC["gg_decay"]**2 * redBR_ZZ
            ggH_tautau = self.redC["gg_decay"]**2 * redBR_tautau
            ggH_inv = self.redC["gg_decay"]**2 * self.extraBR["invisible"]


        else:
            ggH_bb = self.redC["gg_prod_lhc8"]**2 * redBR_bb
            ggH_GaGa = self.redC["gg_prod_lhc8"]**2 * redBR_GaGa
            ggH_WW = self.redC["gg_prod_lhc8"]**2 * redBR_WW
            ggH_ZZ = self.redC["gg_prod_lhc8"]**2 * redBR_ZZ
            ggH_tautau = self.redC["gg_prod_lhc8"]**2 * redBR_tautau
            ggH_inv = self.redC["gg_prod_lhc8"]**2 * self.extraBR["invisible"]


        ttH_WW = self.redC["tt"]**2 * redBR_WW
        WH_ZZ = self.redC["WW"]**2 * redBR_ZZ
        ZH_WW = self.redC["ZZ"]**2 * redBR_WW
        WH_GaGa = self.redC["WW"]**2 * redBR_GaGa
        VBF_GaGa = self.redC["VBF"]**2 * redBR_GaGa
        ttH_GaGa = self.redC["tt"]**2 * redBR_GaGa
        ttH_bb = self.redC["tt"]**2 * redBR_bb
        WH_bb = self.redC["WW"]**2 * redBR_bb
        ttH_tautau = self.redC["tt"]**2 * redBR_tautau
        ZH_inv = self.redC["ZZ"]**2 * self.extraBR["invisible"]
        WH_inv = self.redC["WW"]**2 * self.extraBR["invisible"]
        VBF_inv = self.redC["VBF"]**2 * self.extraBR["invisible"]
        ttH_inv = self.redC["tt"]**2 * self.extraBR["invisible"]
        VBF_tautau = self.redC["VBF"]**2 * redBR_tautau
        WH_tautau = self.redC["WW"]**2 * redBR_tautau
        ttH_ZZ = self.redC["tt"]**2 * redBR_ZZ
        ZH_GaGa = self.redC["ZZ"]**2 * redBR_GaGa
        VBF_bb = self.redC["VBF"]**2 * redBR_bb
        VBF_ZZ = self.redC["VBF"]**2 * redBR_ZZ
        ZH_tautau = self.redC["ZZ"]**2 * redBR_tautau
        ZH_bb = self.redC["ZZ"]**2 * redBR_bb
        VBF_WW = self.redC["VBF"]**2 * redBR_WW
        WH_WW = self.redC["WW"]**2 * redBR_WW
        ZH_ZZ = self.redC["ZZ"]**2 * redBR_ZZ
        
        
        mu = {}
        
        mu[('ggH','gammagamma')] = ggH_GaGa
        mu[('ggH','WW')] = ggH_WW
        mu[('ggH','ZZ')] = ggH_ZZ
        mu[('ggH','bb')] = ggH_bb
        mu[('ggH','tautau')] = ggH_tautau
        mu[('ggH','invisible')] = ggH_inv
        
        mu[('VBF','gammagamma')] = VBF_GaGa
        mu[('VBF','WW')] = VBF_WW
        mu[('VBF','ZZ')] = VBF_ZZ
        mu[('VBF','bb')] = VBF_bb
        mu[('VBF','tautau')] = VBF_tautau
        mu[('VBF','invisible')] = VBF_inv
        
        mu[('WH','gammagamma')] = WH_GaGa
        mu[('WH','WW')] = WH_WW
        mu[('WH','ZZ')] = WH_ZZ
        mu[('WH','bb')] = WH_bb
        mu[('WH','tautau')] = WH_tautau
        mu[('WH','invisible')] = WH_inv
        
        mu[('ZH','gammagamma')] = ZH_GaGa
        mu[('ZH','WW')] =ZH_WW
        mu[('ZH','ZZ')] = ZH_ZZ
        mu[('ZH','bb')] = ZH_bb
        mu[('ZH','tautau')] = ZH_tautau
        mu[('ZH','invisible')] = ZH_inv
        
        mu[('ttH','gammagamma')] = ttH_GaGa
        mu[('ttH','WW')] = ttH_WW
        mu[('ttH','ZZ')] = ttH_ZZ
        mu[('ttH','bb')] = ttH_bb
        mu[('ttH','tautau')] = ttH_tautau
        mu[('ttH','invisible')] = ttH_inv
        
        return mu
