from ReducedCouplingsLO import *
from ReducedCouplingsNNLO import *


class ComputeReducedCouplings:
    
    
    def __init__(self, redC, precision, mh, LO_FF, redC_functions, grid_interpolation):
        
        self.redC = redC
        self.precision = precision
        self.mh = mh
        self.LO_FormFactors = LO_FF
        self.redC_functions = redC_functions
        self.grid_interpolation = grid_interpolation
        self.new_redC = self.GetCouplings()
    
        
    def GetCouplings(self):

        redC_new = {}
        CZ = self.redC["ZZ"].real
        CW = self.redC["WW"].real
        CB = self.redC["bb"].real
        CC = self.redC["cc"].real
        CT = self.redC["tt"].real
        CL = self.redC["tautau"].real
        
        if self.precision == "LO":
            CTIM = self.redC["tt"].imag
            CCIM = self.redC["cc"].imag
            CBIM = self.redC["bb"].imag
            CLIM = self.redC["tautau"].imag
        
        LO_FF = self.LO_FormFactors
        grid_interp = self.grid_interpolation
        addLoopContribution = 0
        

        if "gammagamma" not in self.redC:
            if self.precision == "LO":
                CGaGa = self.redC_functions["redCgammagamma_LO"](CT, CB, CC, CL, CW, CTIM, CBIM, CLIM, CCIM, addLoopContribution, LO_FF)
            else:
                CGaGa = self.redC_functions["redCgammagamma_NNLO"](CT, CB, CC, CL, CW, addLoopContribution, grid_interp)
            redC_new["gammagamma"] = CGaGa

        if "Zgamma" not in self.redC:
            if self.precision == "LO":
                CZGa = self.redC_functions["redCZgamma_LO"](CT, CB, CC, CL, CW, CTIM, CBIM, CCIM, CLIM, addLoopContribution, LO_FF)
            else:
                CZGa = self.redC_functions["redCZgamma_NNLO"](CT, CB, CC, CL, CW, addLoopContribution, grid_interp)
            redC_new["Zgamma"] = CZGa

        if "gg_decay" not in self.redC:
            if self.precision == "LO":
                CGG = self.redC_functions["redCgg_LO"](CT, CB, CC, CTIM, CBIM, CCIM, addLoopContribution, LO_FF)
                CggF = self.redC_functions["redCggF_LO"](CT, CB, CC, CTIM, CBIM, CCIM, addLoopContribution, LO_FF)
                redC_new["gg_prod_LO"] = CggF
            else:
                CGG = self.redC_functions["redCgg_NNLO"](CT, CB, CC, addLoopContribution, grid_interp)
                CggF_LHC8 = self.redC_functions["redCggF_NNLO_LHC8"](CT, CB, grid_interp)
                CggF_Tev = self.redC_functions["redCggF_NNLO_Tev"](CT, CB, grid_interp)
                redC_new["gg_prod_lhc8"] = CggF_LHC8
                redC_new["gg_prof_tevatron"] = CggF_Tev
            redC_new["gg_decay"] = CGG

        if "VBF" not in self.redC:
            if self.precision == "LO":
                CVBF = self.redC_functions["redCVBF_LO"](CW, CZ, grid_interp)
            else:
                CVBF = self.redC_functions["redCVBF_NLO"](CW, CZ, grid_interp)
            redC_new["VBF"] = CVBF


        return redC_new
